(function() {
  var BufferStream, cli, express, fs, path, spawn;

  fs = require('fs');

  cli = require('cli');

  path = require('path');

  express = require('express');

  spawn = require('child_process').spawn;

  BufferStream = require('../buffer-stream');

  cli.enable('status');

  cli.setUsage("benchmark [OPTIONS] streamed_filename");

  cli.parse({
    port: ['p', 'Listen on this port', 'number', 3000]
  });

  cli.main(function(args, opts) {
    var filename;
    var _this = this;
    process.on('uncaughtException', function(err) {
      return _this.fatal(err);
    });
    if (args.length !== 1) this.fatal("path to file required!");
    filename = cli.getPath(args[0]);
    return fs.stat(filename, function(err, stats) {
      var server;
      if (err) _this.fatal(err);
      _this.info("serving " + filename + " (" + stats.size + ") …");
      server = express.createServer();
      server.get('/', function(req, res) {
        var buffer, file, length;
        _this.ok("start streaming …");
        res.header('Content-Length', stats.size);
        cli.progress(length = 0);
        buffer = new BufferStream({
          size: 'flexible',
          blocking: false
        });
        file = spawn('cat', [filename]);
        _this.info("buffer has " + buffer.size + " size.");
        setTimeout(function() {
          _this.info("set buffer size to none (buffer.length=" + (buffer != null ? buffer.length : void 0) + ")");
          return buffer != null ? buffer.setSize('none') : void 0;
        }, 500);
        buffer.on('data', function(chunk) {
          length += chunk.length;
          return cli.progress(length / stats.size);
        });
        buffer.on('error', function(err) {
          return _this.error("buffer errored: " + err);
        });
        res.on('error', function(err) {
          return _this.error("res errored: " + err);
        });
        file.on('exit', function(code) {
          return _this.debug("file.stdout exited with code " + code + ".");
        });
        _this.debug("start piping.");
        buffer.pipe(res);
        return file.stdout.pipe(buffer);
      });
      return server.listen(opts.port, 'localhost', function() {
        _this.ok("server listen on port " + opts.port + " …");
        return _this.info("you can now test it with: wget -Otest http://localhost:" + opts.port + "/");
      });
    });
  });

}).call(this);
